from retrieve_image_data import RtrvData as Data
import location
import image_restriction_main
Google_API = "AIzaSyD0SIrsNBNbE9-hSnfa6gMHALCdLZWJ6uI"

def test_location_all():
        filepath = input("Input File Path for Test Image: ")
        testspassed  = 0
        if(test_get_coord(filepath)==0):
                print "test_get_coord passed\n"
                testspassed+=1
        else:
                print "test_get_coord failed\n"

        if(test_coord_to_zip(filepath)==0):
                print "test_coord_to_zip passed\n"
                testspassed+=1
        else:
                print "test_coord_to_zip failed\n"

        if(test_zip_to_coord(filepath)==0):
                print "test_zip_to_coord passed\n"
                testspassed+=1
        else:
                print "test_zip_to_coord failed\n"

        if(test_sun_position(filepath)==0):
                print "test_sun_position passed\n"
                testspassed+=1
        else:
                print "test_sun_position failed\n"

        print "Number of Tests passed "+ str(testspassed) +"/4\n"


def test_get_coord(filename):
        if(image_restriction_main.check_image(filename)):
                imagedata = Data(filename)
                exifData = imagedata.get_exif(filename, True, False)
                coords = location.get_coord(exifData)
                print "\tOriginal image data: ("+exifData['gps gpslatitude'][1:-1]+","+exifData['gps gpslongitude'][1:-1]+")\n"
                print "\tConverted image data: ("+str(coords[0])+","+str(coords[1])+")\n"
                return 1
                
        else:
                print "\tImage failed a restriction test.\n"
                return 0

def test_coord_to_zip(filename):
        if(image_restriction_main.check_image(filename)):
                imagedata = Data(filename)
                exifData = imagedata.get_exif(filename, True, False)
                coords = location.get_coord(exifData)
                zipcode = location.coord_to_zip(coords,Google_API)
                print "\tOriginal image data: ("+exifData['gps gpslatitude'][1:-1]+","+exifData['gps gpslongitude'][1:-1]+")\n"
                if(zipcode=="99999"):
                        print "\tError fetching ZIP Code\n"
                        return 0
                else:
                        print "\tFetched image data: ZIP = "+zipcode+"\n"
                        return 1 
        else:
                print "\tImage failed a restriction test.\n"
                return 0

def test_zip_to_coord(filename):
        if(image_restriction_main.check_image(filename)):
                imagedata = Data(filename)
                exifData = imagedata.get_exif(filename, True, False)
                coords = location.get_coord(exifData)
                zipcode = location.coord_to_zip(coords,Google_API)
                print "\tOriginal image data: ("+exifData['gps gpslatitude'][1:-1]+","+exifData['gps gpslongitude'][1:-1]+")\n"
                secondcoords = location.zip_to_coord(zipcode,Google_API)
                print "\tOriginal Coordinates: (" + str(coords[0]) + "," + str(coords[1]) + ")\n"
                print "\tFetched Coordinates: (" + str(secondcoords[0]) + "," + str(secondcoords[1]) + ")\n"
                if(secondcoords==(0.0,0.0)):
                        print "\tError fetching Coordinates\n"
                        return 0
                else:
                        if( ( (coords[0] > (secondcoords[0]+0.5)) | (coords[0] < (secondcoords[0]-0.5))) & ( (coords[1] > (secondcoords[1]+0.5)) | (coords[1] < (secondcoords[1]-0.5) ) )  ):
                                print "\tFetched Coordinates are not within +-0.5 accuracy to Original Coordinates"
                                return 0
                        else:
                                print "\tFetched Coordinates are within +-0.5 accuracy to Original Coordinates"
                                return 1
        else:
                print "Image failed a restriction test.\n"
                return 0

def test_sun_position(filename):
        if(image_restriction_main.check_image(filename)):
                imagedata = Data(filename)
                exifData = imagedata.get_exif(filename, True, False)
                print "\tThe sun's position indicates it's : "+location.sun_position(exifData)
                return 1
        else:
                print "\tImage failed a restriction test.\n"
                return 0




import urllib2
import json
import sys
import os

#def get_wunderkey()

def get_data(coord):
    '''
    Purpose:        The purpose of this script is to retrieve meteorological data 
    of a given comma-separated latitude and longitute coordinates via the
    wunderground API.
    Inputs:         coord: string representing comma-separated coordinates.
    Outputs:        weatherData: tuple of city, country, temp in F, sunrise time, sunset time.
    Returns:        dictionary with 5 keys.
    Assumptions:    The wunderground API key is valid.
    '''

    name = coord + '.json'

    try:
        f = urllib2.urlopen('http://api.wunderground.com/api/3b566c79c371f7f4/geolookup/conditions/q/' + name)
        m = urllib2.urlopen('http://api.wunderground.com/api/3b566c79c371f7f4/astronomy/q/' + name);

        conditions = f.read()
        parsedConditions = json.loads(conditions)
        astronomy = m.read()
        parsedAstronomy = json.loads(astronomy)

        city = parsedConditions['location']['city']
        country = parsedConditions['location']['country']
        temp = parsedConditions['current_observation']['temp_f']
        sunriseMin = parsedAstronomy['moon_phase']['sunrise']['minute']
        sunriseHr = parsedAstronomy['moon_phase']['sunrise']['hour']
        sunrise = sunriseHr + ":" + sunriseMin
        sunsetMin = parsedAstronomy['moon_phase']['sunset']['minute']
        sunsetHr = parsedAstronomy['moon_phase']['sunset']['hour']
        sunset = sunsetHr + ":" + sunsetMin

        weatherData = {'city': city, 'country': country, 'temp': temp,
            'sunrise': sunrise, 'sunset': sunset}

    except Exception:
        print "Unable to retrieve data: ", sys.exc_info()[0]
        weatherData = None

    finally:
        return weatherData



version = "0.0.0.7"
#import image_restriction_functions
#import image_restriction_main
#import retrieve_image_data
from wunderData import *
from image_restriction_functions import *
from location import *
from retrieve_image_data import * 
# from horizon import *



import urllib2
import json
import sys
import os
import wunderData

def get_coord(exifdict):
    '''
    Purpose:        The purpose of this script is to extract the Latitude and Longitude from the EXIF data
    Inputs:         exifdict: structure storing the image's EXIF data.
    Outputs:        coords: A tuple of the Latitude and Longitude in Decimal form
    Returns:        (lat,lon)
    Assumptions:    The EXIF data is valid.
    '''
    values = exifdict['gps gpslatitude'][1:-1].split(", ")
    s = values[2]
    df = float(values[0])
    mf = float(values[1])
    smath = s.split("/")
    sf = float(smath[0])/float(smath[1])
    lat = df + mf/60 + sf/3600
    if exifdict['gps gpslatituderef'] == 'S':
        lat = lat*(-1)

    values = exifdict['gps gpslongitude'][1:-1].split(", ")
    s = values[2]
    df = float(values[0])
    mf = float(values[1])
    smath = s.split("/")
    sf = float(smath[0])/float(smath[1])
    lon = df + mf/60 + sf/3600
    if exifdict['gps gpslongituderef'] == 'W':
        lon = lon*(-1)

    return (lat,lon)



def coord_to_zip(coord,googlegeokey):
    '''
    Purpose:        The purpose of this script is to convert Latitude and Longitude to a ZIP Code
    Inputs:         coord: tuple holding latitude and longitude, googlegeokey: The Google geocoding API
    Outputs:        string of 5 digit long ZIP code.
    Returns:        zipcode
    Assumptions:    The EXIF data is valid.
    '''

    try:
        url = "https://maps.googleapis.com/maps/api/geocode/json?latlng="+str(coord[0])+","+str(coord[1])+"&key="+googlegeokey
        c = urllib2.urlopen(url)
        response = c.read()
        parsedResults = json.loads(response)
        zipcode = parsedResults['results'][0]['address_components'][-1]['long_name']

    except Exception:
        print "Unable to retrieve data: ", sys.exc_info()[0]
        zipcode = "99999"

    finally:
        return zipcode



def zip_to_coord(zipcode,googlegeokey):
    '''
    Purpose:        The purpose of this script is to convert ZIP Code to a Latitude and Longitude
    Inputs:         zipcode: 5 digit long ZIP code.
    Outputs:        coord: tuple holding latitude and longitude
    Returns:        (lat,lon)
    Assumptions:    The EXIF data is valid.
    '''
    try:
        url = 'https://maps.googleapis.com/maps/api/geocode/json?address='+zipcode+'&key='+googlegeokey
        c = urllib2.urlopen(url)
        results = c.read()
        parsedResults = json.loads(results)
	lat = float(parsedResults['results'][0]['geometry']['location']['lat'])
        lon = float(parsedResults['results'][0]['geometry']['location']['lng'])

    except Exception:
        print "Unable to retrieve data: ", sys.exc_info()[0]
        (lat,lon) = (0.0,0.0)

    finally:
        return (lat,lon)


def sun_position(exifdict):
    '''
    Purpose:        Identify whether an image was taken during sunrise or sunset.
    Inputs:         exifdict: structure storing the image's EXIF data.
    Outputs:        string
    Returns:        sunrise,sunset,night,day
    Assumptions:    N/A
    '''
    coord = get_coord(exifdict)
    wData = wunderData.get_data(str(coord[0])+","+str(coord[1]))
    sunriseTime = wData['sunrise'].split(':')
    sunsetTime = wData['sunset'].split(':')
    sunriseTarget = (int(sunriseTime[0])*60)+int(sunriseTime[1])
    sunsetTarget = (int(sunsetTime[0])*60)+int(sunsetTime[1])

    hoursTime = (str(exifdict['exif datetimeoriginal']).split(' '))[1].split(':')
    pictureTime = (int(hoursTime[0])*60)+int(hoursTime[1])+int(float(hoursTime[2])/60)

    if ((pictureTime >= (sunriseTarget - 15)) & (pictureTime <= (sunriseTarget + 30))):
        return 'sunrise'
    elif ((pictureTime >= (sunsetTarget - 15)) & (pictureTime <= (sunsetTarget + 30))):
        return 'sunset'
    elif ((pictureTime > (sunsetTarget + 15))|(pictureTime < (sunriseTarget - 15))):
        return 'night'
    else:
        return 'day'





# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os.path
from setuptools import find_packages, setup

# Package data
# ------------
_author = 'Aerolyzer Developers'
_author_email = 'aerolyzer@googlegroups.com'
_classifiers = [
    'Environment :: Console',
    'Intended Audience :: Developers',
    'Intended Audience :: Information Technology',
    'Intended Audience :: Science/Research',
    'Topic :: Scientific/Engineering',
    'Development Status :: 1 - Planning',
    'License :: OSI Approved :: Apache Software License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Software Development :: Libraries :: Python Modules',
]
_description = 'Aerolyzer analyzes images to infer atmospheric composition'
_download_url = 'https://github.com/Aerolyzer/Aerolyzer/archive/0.0.0.7.tar.gz'
_requirements = ['exifread', 'numpy>=1.13.3', 'opencv-python>=3.3.0', 'pyyaml']
_pyrequirements = '>=2.7.12'
_keywords = ['atmospheric composition', 'sunsets', 'image analysis', 'aerosols']
_license = 'Apache License, Version 2.0'
_long_description = 'A Python suite for analyzing images to infer aerosol types in the image.'
_name = 'aerolyzer'
_namespaces = []
_packages = ['aerolyzer']
_test_suite = 'aerolyzer.tests'
_url = 'https://github.com/Aerolyzer/Aerolyzer'
_version = '0.0.0.7'
_zip_safe = False

# Setup Metadata
# --------------
setup(
    author=_author,
    author_email=_author_email,
    classifiers=_classifiers,
    description=_description,
    download_url=_download_url,
    include_package_data=True,
    install_requires=_requirements,
    python_requires=_pyrequirements,
    keywords=_keywords,
    license=_license,
    long_description=_long_description,
    name=_name,
    namespace_packages=_namespaces,
    packages=_packages,
    test_suite=_test_suite,
    url=_url,
    version=_version,
    zip_safe=_zip_safe,
)



'''
Data Retrieval File
Description: This file contains all functions for retrieving data from an image.
'''

import os
import cv2
import yaml
import exifread
from datetime import datetime

class RtrvData(object):
    'Class containing all image restriction functions'

    def __init__(self, pathPassed):
        self.data = self._import_yaml(os.getcwd() + "/../../Aerolyzer/aerolyzer/config/retrieve_image_data_conf.yaml")
        try:
            os.path.exists(pathPassed)
        except ImportError:
            pass

    '''
    Purpose:        The purpose of this function is to retrieve the EXIF data from an
                    image.
    Inputs:         string pathname
    Outputs:        None
    Returns:        dictionary of EXIF data tags
    Assumptions:    The image's path has been provided
    '''
    def _get_all_exif(self, pathname):
        img = open(pathname, 'rb')
        tags = exifread.process_file(img)
        return tags

    '''
    Purpose:        The purpose of this function is to retrieve the EXIF data from an
                    image.
    Inputs:         string pathname
                    bool setTypes toggles conversion of types
                    bool dateToString toggles conversion of dateToString
    Outputs:        None
    Returns:        dictionary of EXIF data tags
    Assumptions:    The image's path has been provided
    '''
    def get_exif(self, pathname, setTypes, dateToString):
        tags = {};
        allTags = self._get_all_exif(pathname)
        for key, value in allTags.iteritems():
            for entry in self.data['selectTags']:
                if (key == entry):
                    tags[key.lower()] = value
        if setTypes is True:
            tags = self._set_types(tags, dateToString)
        tags['file size'] = self._get_file_size(pathname)
        tags['file type'] = self._get_file_type(pathname).lower()
        return tags

    '''
    Purpose:        The purpose of this function is to retrieve the RGB values from an
                    image.
    Inputs:         string pathname
    Outputs:        None
    Returns:        tuple of lists of lists containing RGB values
    Assumptions:    The image's path has been provided
    '''
    def get_rgb(self, pathname):
        img = cv2.imread(pathname,1)
        return img

    '''
    Purpose:        The purpose of this function is to import the contents of the configuration file.
    Inputs:         string conf_file
    Outputs:        None
    Returns:        reference to configuration file
    Assumptions:    N/A
    '''
    def _import_yaml(self, confFile):
        assert (type(confFile) == str), "configuration file not passed as a string"
        with open(confFile, 'r') as file:
            doc = yaml.load(file)
            file.close()
        return doc

    '''
    Purpose:        The purpose of this function is to determine the size of the image
    Inputs:         string pathname
    Outputs:        None
    Returns:        int size of file in bytes
    Assumptions:    N/A
    '''
    def _get_file_size(self, pathname):
        return os.path.getsize(pathname)

    '''
    Purpose:        The purpose of this function is to determine the type of the image
    Inputs:         string pathname
    Outputs:        None
    Returns:        string file type (.ext)
    Assumptions:    N/A
    '''
    def _get_file_type(self, pathname):
        filename, fileExtension = os.path.splitext(pathname)
        return fileExtension

    '''
    Purpose:        The purpose of this function is to set the tag values to the correct
                    data type
    Inputs:         string pathname
                    bool dateToString toggles conversion of datetimeTags to strings
    Outputs:        None
    Returns:        string file type (.ext)
    Assumptions:    N/A
    '''
    def _set_types(self, tags, dateToString):
        for key, value in tags.iteritems():
            for entry in self.data['stringTags']:
                if(key == entry):
                    tags[key] = str(value)
            for entry in self.data['datetimeTags']:
                if(key == entry):
                    stringDate = str(value)
                    if dateToString is True:
                        tags[key] = stringDate
                    else:
                        tags[key] = datetime.strptime(stringDate, '%Y:%m:%d %H:%M:%S')
            for entry in self.data['intTags']:
                if(key == entry):
                    strKey = str(value)
                    tags[key] = int(strKey)
        return tags



'''
Image Restriction Main File
Description: This file is called by the website when a photo is uploaded, and uses
information from image_restriction_functions and image_restrictions_conf to verify that
all image restrictions have been met.
'''

import sys

from image_restriction_functions import imgRestFuncs as Fxn
from retrieve_image_data import RtrvData as Data

'''
Purpose:        The purpose of this function is to assert each restriction check
Inputs:         dict exifData, object functions
Outputs:        restriction check results
Returns:        N/A
Assumptions:    N/A
'''
def program(fxn, data, exifData, pathname):
    if not 'image model' in exifData:
        isVerified = {'meetsRest': False, 'error_message': "The image must be a mobile image from a supported device"}
        return isVerified
    if not fxn.is_device(exifData['image model']):
        isVerified = {'meetsRest': False, 'error_message': "The image must be a mobile image from a supported device"}
        return isVerified
    if not fxn.is_edited(exifData['image datetime'], exifData['exif datetimeoriginal']):
        isVerified = {'meetsRest': False, 'error_message': "The image cannot be edited or filtered in any way"}
        return isVerified
    if not fxn.is_landscape(data.get_rgb(pathname)):
        isVerified = {'meetsRest': False, 'error_message': "The image must be of a direct landscape with a sky and view"}
        return isVerified
    if not fxn.is_size(exifData['file size']):
        isVerified = {'meetsRest': False, 'error_message': "The image must be no larger than 4Mb"}
        return isVerified
    if not fxn.is_type(exifData['file type']):
        isVerified = {'meetsRest': False, 'error_message': "The file type of the image must be .jpg or .png"}
        return isVerified
    if not fxn.is_res(exifData['exif exifimagewidth'], exifData['exif exifimagelength']):
        isVerified = {'meetsRest': False, 'error_message': "The image must be in the resolution range 600X600-6000X6000"}
        return isVerified
    if not 'gps gpslatitude' in exifData or not 'gps gpslongitude' in exifData:
        isVerified = {'meetsRest': False, 'error_message': "Location services must be enabled for the camera"}
        return isVerified
    locExifData = data.get_exif(pathname, False, False)
    exifData = data.get_exif(pathname, True, True)
    isVerified = {'meetsRest': True, 'exifData': exifData, 'locExifData': locExifData}
    return isVerified

'''
Purpose:        The purpose of this main function is to check all image restrictions
                for use in the Aerolyzer app.
Inputs:         string filename of image locally
Outputs:        None
Returns:        N/A
Assumptions:    N/A
'''
def check_image(filename):
    #instantiate classes
    fxn     = Fxn()
    #Retrieve exif data
    data    = Data(filename)
    exifData = data.get_exif(filename, True, False)
    isVerified = program(fxn, data, exifData, filename)
    return isVerified

'''
Purpose:        The purpose of this main function is to check all image restrictions and
                produce the correct error message should one occur.
Inputs:         string image (as sys.argv[1])
Outputs:        None
Returns:        N/A
Assumptions:    N/A
'''
def main():
    #instantiate classes
    fxn     = Fxn()

    #Retrieve exif data
    if(len(sys.argv) < 2):
        #use default image
        data    = Data("./images/good-images/1484949760_19_2615.jpg")
        exifData = data.get_exif("./images/good-images/1484949760_19_2615.jpg", True, True)
        program(fxn, data, exifData, "./images/good-images/1484949760_19_2615.jpg")
    elif(len(sys.argv) == 2):
        data    = Data(sys.argv[1])
        exifData = data.get_exif(sys.argv[1], True, False)
        program(fxn, data, exifData, sys.argv[1])
    elif(len(sys.argv) > 2):
        #error
        print "Please pass only 1 image to this program as an argument"

if __name__ == '__main__':
    main()



'''
Image Restriction Function File
Description: This file contains all functions for the verifying image restrictions.
'''
import os
import re
import cv2
import yaml
from datetime import datetime
import exifread
import numpy as np

class imgRestFuncs(object):
    'Class containing all image restriction functions'
    '''
    Purpose:        sigmoid function that takes in a value and returns a value from 0 to 1
    Inputs:         float
    Outputs:        None
    Returns:        Float between 0, 1
    Assumptions:    N/A
    '''
    def sigm(self, x):
        return 1 / (1 + np.exp(-x))

    def __init__(self):
        self.criteria = self._import_yaml(os.getcwd() + "/../../Aerolyzer/aerolyzer/config/image_restriction_conf.yaml")

    '''
    Purpose:        The purpose of this function is to determine whether or not the device the
                    image was taken on is an accepted mobile device.
    Inputs:         string device
    Outputs:        None
    Returns:        Boolean
    Assumptions:    N/A
    '''
    def is_device(self, device):
        if device in self.criteria['acceptedMobileDevices']:
            return True
        else:
            return False


    '''
    Purpose:        The purpose of this function is to determine whether or not the image was
                    altered from its original form. I.e. do the modification and creation dates coincide.
    Inputs:         datetime created, datetime modified
    Outputs:        None
    Returns:        Boolean
    Assumptions:    N/A
    '''
    def is_edited(self, modified, created):
        if (created == modified):
            return True
        else:
            return False

    '''
    Purpose:        The purpose of this function is to determine whether or not the image
                    contains a direct landscape with sky and view.
    Inputs:         tuple lists of lists of rgb values (red, green, blue)
    Outputs:        None
    Returns:        Boolean
    Assumptions:    N/A
    '''
    def is_landscape(self, img):
        # Create a mask
        #mask = np.zeros(img.shape[:2], np.uint8)
        #mask[0:(img.shape[0]/2), 0:img.shape[1]] = 255
        #masked_img = cv2.bitwise_and(img,img,mask = mask)

        # Create histograms with 16 bins in range 0-255
        #hist_blue = cv2.calcHist([img],[0],mask,[16],[0,255])
        #hist_green = cv2.calcHist([img],[1],mask,[16],[0,255])
        #hist_red = cv2.calcHist([img],[2],mask,[16],[0,255])
        return self._is_sky(img)

    '''
    Purpose:        The purpose of this function is to determine whether or not the size of
                    the image is less than or equal to 200kb.
    Inputs:         dict exifData, int imgMaxSize, int imgMaxSizeBytesShort,
                    string fileSize
    Outputs:        None
    Returns:        Boolean
    Assumptions:    N/A
    '''
    def is_size(self, fileSize):
        if(fileSize > self.criteria['imgMaxSizeNumber']):
            return False
        else:
            return True

    '''
    Purpose:        The purpose of this function  is to determine whether or not the image is
                    an accepted file type.
    Inputs:         string fileType
    Outputs:        None
    Returns:        Boolean
    Assumptions:    N/A
    '''
    def is_type(self, fileType):
        if fileType in self.criteria['acceptedFileTypes']:
            return True
        else:
            return False

    '''
    Purpose:        The purpose of this function is to determine whether or not the image
                    exceeds the minimum resolution.
    Inputs:         int imageWidth, int imageLength
    Outputs:        None
    Returns:        Boolean
    Assumptions:    N/A
    '''
    def is_res(self, imageWidth, imageLength):
        if (imageWidth >= self.criteria['imgWidthMin']) and (imageLength >= self.criteria['imgLengthMin']):
            if (imageWidth <= self.criteria['imgWidthMax']) and (imageLength <= self.criteria['imgLengthMax']):
                return True
        else:
            return False

    '''
    Purpose:        The purpose of this function is to determine whether or not the image contains
                    a valid sky or not. 
    Inputs:         image
    Outputs:        None
    Returns:        Boolean
    Assumptions:    N/A
    '''
    def _is_sky(self, img):
        syn0 = np.array([
                        [0.75627016, -0.24375051, -1.01208743, -2.82196649, -2.13659437, 1.44774139, -3.42969981, -2.30166462, 0.16540564, -5.46942492, -0.89061896, -3.05775917, 13.78175927],
                        [4.01459384, 6.80890819, -1.33467051, -8.6509415, -0.97254904, 7.65391215,  2.80460369, -7.96453625, -0.78258749,  0.23794325, -3.05427998, -1.08719948, -1.05505044],
                        [-2.99415349, -5.03765458,  0.95907762, 6.2100258, -0.77418415, -6.45073556, -4.28959853,  6.4269825, 0.08357833, -1.34192387, 2.45324273, -1.88004736, -2.17461887],
                        [-1.33260449, -0.06806694, -0.54152864, 2.29643921, -1.49527328, -0.27464986,  1.61048268,  1.87987677, -0.71722108, -0.31390847, 1.09973296, -0.71580621, -11.78420703],
                        [0.41402219, 0.13775545,  0.17955999, 1.53319054, -0.37451253, -0.19688275, -1.56439065, -0.60001667, -1.31350033, -0.1028686, 0.2385619, -0.98948445, -7.51238318],
                        [-1.28689963, -1.83018296,  0.95150202, 0.35453703, -1.4224101, -1.45727839, -2.11189616,  1.74252963, -1.71482389, -2.29710027, 0.59118241, -0.89214872, -3.12445739]])
        syn1 = np.array([
                        [ -5.93204166],
                        [ -8.12550549],
                        [  0.18872795],
                        [  8.65136128],
                        [ -3.49798356],
                        [ -9.83118546],
                        [ -8.08570886],
                        [  8.19601228],
                        [ -1.44990589],
                        [ -9.531507  ],
                        [  2.54965604],
                        [ -5.09832493],
                        [ 18.47629354]])
        mask = np.zeros(img.shape[:2], np.uint8)
        mask[0:(img.shape[0] / 2), 0:img.shape[1]] = 255
        masked_img = cv2.bitwise_and(img, img, mask = mask)

        # Create histograms with 16 bins in range 0-255
        color = ('b', 'g', 'r')
        b, g, r = cv2.split(img)
        dimy, dimx = img.shape[:2]

        largest = [0, 0]
        it = dimy / 200 #iterations = total number of rows(pixels) / 200
        for i in range(dimy / 6, (dimy / 6) * 5, it):   #only looking at the middle half of the image
            ravg = (sum(r[i]) / float(len(r[i])))
            gavg = (sum(g[i]) / float(len(g[i])))
            bavg = (sum(b[i]) / float(len(b[i])))
            avg = (ravg + gavg + bavg) / 3
            pravg = (sum(r[i - it]) / float(len(r[i - it])))
            pgavg = (sum(g[i - it]) / float(len(g[i - it])))
            pbavg = (sum(b[i - it]) / float(len(b[i - it])))
            pavg = (pravg + pgavg + pbavg) / 3
            diff = pavg - avg
            if diff > largest[0]:   #only getting the largest intensity drop.
                largest = [diff,i-(it/2)]
        sky = img[0:largest[1], 0:dimx]#cropping out landscape
        h1 = sky[0:(sky.shape[0] / 2), 0:dimx]#top half of sky
        h2 = sky[(sky.shape[0] / 2):(sky.shape[0]), 0:dimx]#bottom half
        mask1 = np.zeros(h1.shape[:2], np.uint8)
        mask1[0:(h1.shape[0] / 2), 0:h1.shape[1]] = 255
        hist1 = [0,0,0]
        hist2 = [0,0,0]
        max1 = [0,0,0]
        max2 = [0,0,0]
        for i,col in enumerate(color):
            hist1[i] = cv2.calcHist([h1], [i], mask1, [255], [0, 255])
            max1[i] = np.argmax(hist1[i][6:250])
    
        mask2 = np.zeros(h2.shape[:2], np.uint8)
        mask2[0:(h2.shape[0] / 2), 0:h2.shape[1]] = 255
        for j,col in enumerate(color):
            hist2[j] = cv2.calcHist([h2], [j], mask2, [255], [0, 255])
            max2[j] = np.argmax(hist2[j][6:250])
        X = np.array([float(max1[0])/255., float(max1[1])/255., float(max1[2])/255., float(max2[0])/255., float(max2[1])/255., float(max2[2])/255.])
        l1dup = self.sigm(np.dot(X,syn0))
        l2dup = self.sigm(np.dot(l1dup,syn1))
        if float(l2dup) >= 0.5:
            return True
        return False 


    '''
    Purpose:        The purpose of this function is to import the contents of the configuration file.
    Inputs:         string conf_file
    Outputs:        None
    Returns:        reference to configuration file
    Assumptions:    N/A
    '''
    def _import_yaml(self, confFile):
        with open(confFile, 'r') as f:
            doc = yaml.load(f)
            f.close()
        return doc
